﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.ComponentModel.DataAnnotations;
using MVVMTutorial.Web.Models;

namespace MVVMTutorial.ViewModels
{
    [System.Diagnostics.DebuggerStepThroughAttribute()]
    public class LoginViewModel : System.ComponentModel.INotifyPropertyChanged
    {
        #region 字段
        private string ValidationCodeField;

        private string GivenValidationCodeField;

        private string MessageField;

        private bool IsDoneField;

        private LoginProxy.LoginServiceClient client;
        #endregion
        
        #region 属性
        public User Data
        {
            get; private set;
        }

        [Required]
        [StringLength(4, MinimumLength = 4)]
        [Display(Name = "校验码", Description = "不区分大小写")]
        [RegularExpression("^[a-zA-Z0-9]*$", ErrorMessage = "只能输入字母、数字")]
        public string ValidationCode
        {
            get
            {
                return this.ValidationCodeField;
            }
            set
            {
                if ((object.ReferenceEquals(this.ValidationCodeField, value) != true))
                {
                    this.ValidateProperty("ValidationCode", value);
                    if (!object.Equals(value, this.GivenValidationCodeField))
                    {
                        throw new NotSupportedException("请按照给定校验码输入");
                    }
                    this.ValidationCodeField = value;
                    this.RaisePropertyChanged("ValidationCode");
                }
            }
        }
        public string GivenValidationCode
        {
            get
            {
                return this.GivenValidationCodeField;
            }
            set
            {
                if ((object.ReferenceEquals(this.GivenValidationCodeField, value) != true))
                {
                    this.GivenValidationCodeField = value;
                    this.RaisePropertyChanged("GivenValidationCode");
                }
            }
        }
        /// <summary>
        /// 用于代表消息，包括异常
        /// </summary>
        public string Message
        {
            get
            {
                return this.MessageField;
            }
            set
            {
                if ((object.ReferenceEquals(this.MessageField, value) != true))
                {
                    this.MessageField = value;
                    this.RaisePropertyChanged("Message");
                }
            }
        }
        /// <summary>
        /// 用于表示登录是否成功
        /// </summary>
        public bool IsDone
        {
            get
            {
                return this.IsDoneField;
            }
            set
            {
                if ((object.ReferenceEquals(this.IsDoneField, value) != true))
                {
                    this.IsDoneField = value;
                    this.RaisePropertyChanged("IsDone");
                }
            }
        }
        #endregion

        #region 构造函数
        public LoginViewModel()
        {
            this.Data = new User();
        }
        #endregion

        #region 非公开的方法
        /// <summary>
        /// 检查是否要初始化代理
        /// </summary>
        /// <returns></returns>
        bool NeedInitializeClient()
        {
            return this.client == null ||
                this.client.State == System.ServiceModel.CommunicationState.Closed ||
                this.client.State == System.ServiceModel.CommunicationState.Faulted;
        }
        /// <summary>
        /// 初始化代理
        /// </summary>
        void InitializeClient()
        {
            // 使用默认的 Binding 和 EndpointAddress 初始化，在实际项目应使用 
            // new LoginProxy.LoginServiceClient(binding,remoteAddress)
            this.client = new LoginProxy.LoginServiceClient();

            this.client.GenerateValidationCodeCompleted += (sender, e) =>
            {
                if (e.Error == null)
                {
                    this.GivenValidationCode = e.Result;
                }
                else
                {
                    // 处理异常
                    this.HandleException(e.Error);
                }
            };

            this.client.LoginCompleted += (sender, e) =>
            {
                if (e.Error == null)
                {
                    if (e.Result == 1)
                    {
                        // 登录成功
                        this.HandleMessage("登录成功");
                        this.IsDone = true;
                    }
                    else if (e.Result == 0)
                    {
                        // 用户名或密码错误
                        this.HandleMessage("用户名或密码错误");
                    }
                    else if (e.Result == 4)
                    {
                        // 校验码失效
                        this.HandleMessage("校验码失效");
                    }
                }
                else
                {
                    // 处理异常
                    this.HandleException(e.Error);
                }
            };
        }
        /// <summary>
        /// 简单的消息处理
        /// </summary>
        /// <param name="msg"></param>
        void HandleMessage(string msg)
        {
            this.Message = string.Format("消息：{0}", msg);
        }
        /// <summary>
        /// 简单的异常处理
        /// </summary>
        /// <param name="ex"></param>
        void HandleException(Exception ex)
        {
            this.Message = string.Format("异常：{0}" , ex.Message);
        }
        protected void ValidateProperty(string memberName, object value)
        {
            Validator.ValidateProperty(value, new ValidationContext(this, null, null) { MemberName = memberName });
        }
        protected void RaisePropertyChanged(string propertyName)
        {
            System.ComponentModel.PropertyChangedEventHandler propertyChanged = this.PropertyChanged;
            if ((propertyChanged != null))
            {
                propertyChanged(this, new System.ComponentModel.PropertyChangedEventArgs(propertyName));
            }
        }
        #endregion

        #region 公开的方法
        public void GenerateValidationCodeAsync()
        {
            if (this.NeedInitializeClient())
            {
                this.InitializeClient();
            }
            this.client.GenerateValidationCodeAsync();
        }

        public void LoginAsync()
        {
            if (this.NeedInitializeClient())
            {
                this.InitializeClient();
            }
            this.client.LoginAsync(this.Data, this.ValidationCode);
        }
        #endregion

        public event System.ComponentModel.PropertyChangedEventHandler PropertyChanged;


    }
}
